package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.mapper.QmsQueueListMapper;
import com.steerinfo.dil.model.QmsQueueList;
import com.steerinfo.dil.util.DataChange;
import com.steerinfo.dil.model.QmsQueueResult;
import com.steerinfo.dil.mapper.QmsQueueResultMapper;
import com.steerinfo.dil.service.IQmsQueueResultService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.math.BigDecimal;

/**
 * QmsQueueResult服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-14 11:27
 * 类描述
 * 修订历史：
 * 日期：2021-09-14
 * 作者：generator
 * 参考：
 * 描述：QmsQueueResult服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "qmsQueueResultService")
public class QmsQueueResultServiceImpl implements IQmsQueueResultService {

    @Autowired
    private QmsQueueResultMapper qmsQueueResultMapper;

    @Autowired
    private QmsQueueListMapper qmsQueueListMapper;

    /**
     * 新增排队实绩
     * @param map {orderNumber}运输订单号
     * @return
     */
    @Override
    public int addQueueResult(Map<String, Object> map) {
        QmsQueueResult qmsQueueResult = new QmsQueueResult();
        //生成主键
        long id;
        BigDecimal selectMaxId = qmsQueueResultMapper.selectMaxId();
        id = selectMaxId == null ? 1 : selectMaxId.longValue() + 1;
        qmsQueueResult.setId(new BigDecimal(id));
        //获取传入运输订单号
        String orderNumber = (String) map.get("orderNumber");
        //查询实绩总表ID与运力Id
        Map<String, Object> cidAndTidMap = qmsQueueResultMapper.queryCIdAndTIdByOrderNumber(orderNumber);
        BigDecimal capacityId = DataChange.dataToBigDecimal(cidAndTidMap.get("capacityId"));
        map.put("capacityId", capacityId);  //使用传进来的map将数据一路带到尾
        qmsQueueResult.setCapacityId(capacityId);
        BigDecimal resultTotalId = DataChange.dataToBigDecimal(cidAndTidMap.get("resultTotalId"));
        qmsQueueResult.setResultTotalId(resultTotalId);
        //通过运输订单号查询物资Id
        BigDecimal materialId = qmsQueueResultMapper.queryOrderMesByOrderId(DataChange.dataToBigDecimal(cidAndTidMap.get("orderId")));
        //通过物资Id查询门岗ID
        map.put("materialId", materialId);
        map.put("type", 0); //匹配物资可进厂的门岗
        List<BigDecimal> gatepostList = qmsQueueResultMapper.queryGatepostByMaterialId(map); //可能会有多个门岗可入
        BigDecimal gatepostId = gatepostList.get(0);
        map.put("gridType", 3);
        map.put("queueLocationId", gatepostId);
        //通过匹配出来的门岗ID获取网格ID
        BigDecimal gridId = qmsQueueResultMapper.getGridIdByGatepostIdAndGridType(map);
        map.put("gridId", gridId);
        //新增进厂排队实绩
        qmsQueueResult.setGridId(gridId);
        //添加申请时间
        qmsQueueResult.setResultApplyforTime(new Date());
        //系统自动下发 添加下发时间
        qmsQueueResult.setResultDownTime(new Date());

        qmsQueueResult.setInsertTime(new Date());
        qmsQueueResult.setInsertUsername("admin");
        return qmsQueueResultMapper.insertSelective(qmsQueueResult);
    }

    /**
     * 查看排队转移界面
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getChangeQueueMes(Map<String, Object> map) {
        return qmsQueueResultMapper.getChangeQueueMes(map);
    }

    /**
     * 查询排队申请
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueApply(Map<String, Object> map) {
        return qmsQueueResultMapper.getQueueApply(map);
    }

    /**
     * 查询执行中运单
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getAllExecutionOrder(Map<String, Object> map) {
        return qmsQueueResultMapper.getAllExecutionOrder(map);
    }

    /**
     * 指令接收 判断条件：是否有排队结束时间
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> orderReceive(Map<String, Object> map) {
        List<Map<String, Object>> orderReceiveList = qmsQueueResultMapper.getOrderReceive(map);
        //遍历数据 通过是否有接收时间 来判断是否接收
        for (Map<String, Object> orderReceiveMap : orderReceiveList) {
            if(orderReceiveMap.get("resultAcceptTime") != null){
                orderReceiveMap.put("isReceive", "是");
            }else {
                orderReceiveMap.put("isReceive", "否");
            }
        }
        return orderReceiveList;
    }

    /**
     * 查询排队开始 数据
     * 可用 gridId 网格ID
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueListByQueueUp(Map<String, Object> map) {
        return qmsQueueResultMapper.getQueueListByQueueUp(map);
    }

    /**
     * 查询排队取消数据
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueCancel(Map<String, Object> map) {
        return qmsQueueResultMapper.getQueueCancel(map);
    }

    /**
     * 通过实绩ID查询排队取消信息
     * @param resultId
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueCancelByResultId(Integer resultId) {
        return qmsQueueResultMapper.getQueueCancelByResultId(resultId);
    }

    /**
     * 修改排队取消实绩
     * @param qmsQueueResult
     * @return
     */
    @Override
    public int updateQueueCancel(QmsQueueResult qmsQueueResult) {
        qmsQueueResult.setUpdateTime(new Date());
        qmsQueueResult.setUpdateUsername("admin");
        return qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
    }

    /**
     * 查询排队插队
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueInsert(Map<String, Object> map) {
        return qmsQueueResultMapper.getQueueInsert(map);
    }

    /**
     * 查询排队结束
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueEndResult(Map<String, Object> map) {
        return qmsQueueResultMapper.getQueueEndResult(map);
    }

    /**
     * 汽车监控
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> capacityMonitor(Map<String, Object> map) {
        List<Map<String, Object>> list = qmsQueueResultMapper.capacityMonitor(map);
        for (Map<String, Object> capacityMap : list) {
            //如果结束时间为空 排队总时长为
            if(capacityMap.get("resultEndTime") == null){
                Date date1 = (Date) capacityMap.get("resultApplyforTime");
                Date date2 = new Date();
                String differenceTime = DataChange.calculatedTimeDifference(date1, date2);
                capacityMap.put("resultQueueAllTime",differenceTime);
            }else {
                //否则排队时长为结束时长 - 申请时长
                Date date1 = (Date)capacityMap.get("resultApplyforTime");
                Date date2 = (Date)capacityMap.get("resultEndTime");
                String differenceTime = DataChange.calculatedTimeDifference(date1, date2);
                capacityMap.put("resultQueueAllTime",differenceTime);
            }
        }
        return list;
    }

    //********************************APP*********************************************

    /**
     * 根据车牌号查询排队下发
     * @param capacityNumber
     * @return
     */
    @Override
    public List<Map<String, Object>> getOrderReceive2(String capacityNumber) {
        BigDecimal gridId = getGridId(capacityNumber);
        if (gridId == null) {
            List<Map<String,Object>> mapList = new ArrayList<>();
            return mapList;
        }
        Map<String,Object> map = new HashMap<>();
        map.put("capacityNumber",capacityNumber);
        map.put("gridId",gridId);
        List<Map<String, Object>> orderReceive = qmsQueueResultMapper.getOrderReceive2(map);
        return orderReceive;
    }

    /**
     * 排队取消
     * @param mapValue
     * @return
     */
    @Override
    public int cancelQueue(Map<String,Object> mapValue) {
        String capacityNumber = (String) mapValue.get("capacityNumber");
        String resultCancelReason = (String) mapValue.get("resultCancelReason");
        BigDecimal resultId = qmsQueueResultMapper.getResultIdByCapacityId(capacityNumber);
        QmsQueueResult qmsQueueResult = qmsQueueResultMapper.selectByPrimaryKey(resultId);
        Map<String, Object> listIdMap = qmsQueueResultMapper.getListIdByResultId(resultId);
        BigDecimal listId = (BigDecimal) listIdMap.get("listId");
        QmsQueueList qmsQueueList = qmsQueueListMapper.selectByPrimaryKey(listId);
        qmsQueueList.setDeleted(new BigDecimal(1));
        qmsQueueResult.setResultCancelTime(new Date());
        qmsQueueResult.setResultCancelReason(resultCancelReason);
        qmsQueueListMapper.updateByPrimaryKeySelective(qmsQueueList);
        return qmsQueueResultMapper.updateByPrimaryKeySelective(qmsQueueResult);
    }


    /**
     * 通过车牌号得到网格id
     * @param capacityNumber
     * @return
     */
    @Override
    public BigDecimal getGridId(String capacityNumber) {
        return qmsQueueResultMapper.getGridIdNotNull(capacityNumber);
    }


    /**
     * 查询各门岗和仓库的排队状态
     * @return
     */
    @Override
    public List<Map<String, Object>> getQueueStatusList() {
        return qmsQueueResultMapper.getQueueStatusList();
    }


}
